/**
 * Introduction
 *
 * @title ROV - Easy applications from C
 *
 * @copyright Jason Tribbeck / 7th ARM 1994-2001
 *
 * @authors Jason Tribbeck
 *
 * @file fontmenu.c
 */

#include <ctype.h>
#include <string.h>

#include "main.h"
#include "canvas.h"
#include "menu.h"
#include "kernel.h"

static unsigned char oldpath[512] = "", sm[256];
static int list[256];
static int entries = 0, old_tick = -1, old_stick = -1;
static char snames[512];
static unsigned char tickitem[80] = "";

static menu fontmenu = 0, submenu = 0;

typedef void (* fontmenu_handlep)(char *);

static fontmenu_handlep p;

static int strincmp(char * a, char * b, int l)
{
  while(l) {
    if((* a == 0) && (* b == 0)) return 0;
    if(toupper(* a) != toupper(* b)) return 1;
    a ++;
    b ++;
    l --;
  }
  return 0;
}

#ifndef NOSTRICMP
static int stricmp(char * a, char * b)
{
  do {
    if((* a == 0) && (* b == 0)) return 0;
    if(toupper(* a) != toupper(* b)) return 1;
    a ++ ;
    b ++ ;
  } while(1);
}
#endif

static void finish(unsigned char * d)
{
  while(* d >= ' ') d ++ ;
  * d = 0;
}

static void update_submenu(int n)
{
  int l, size = 0, ticking;
  unsigned char entry[80], entry2[80], * s = (unsigned char *)snames;
  _kernel_swi_regs regs;

  menu_clear(submenu);
  regs.r[1] = (int)entry;
  regs.r[4] = (int)entry2;
  old_stick = -1;
  for(l = list[n]; l < list[n + 1]; l ++) {
    regs.r[2] = l;
    regs.r[3] = 80;
    regs.r[5] = 80;
    if(_kernel_swi(0x60091, & regs, & regs))
      return;
    finish(entry);
    ticking = !stricmp((char *)entry, (char *)tickitem);
    if(size == 0)
      while((entry[size] != '.') && (entry[size] >= ' '))
	size ++ ;
    if(entry[size] < ' ')
      entry[size] = 0;
    strcpy((char*)s, (char*)entry);
    if(entry[size])
      menu_add(submenu, (char *)s + size + 1);
    else
      menu_add(submenu, "<Normal>");
    if(ticking)
      menu_tick(submenu, old_stick = (l - list[n]), 1);
    s += strlen((char*)s) + 1;
  }
}

static void font_hnd(menu m, int n, menu_list ls[], int rs[])
{
  unsigned char entry[80], entry2[80];
  _kernel_swi_regs regs;

  if(n == -1)
    return;
  regs.r[1] = (int)entry;
  regs.r[4] = (int)entry2;
  regs.r[2] = list[n];
  if(rs[0] != -1)
    regs.r[2] += rs[0];
  regs.r[5] = 80;
  regs.r[3] = 80;
  if(_kernel_swi(0x60091, & regs, & regs))
    return;
  finish(entry);
  if(p) p((char *)entry);
}

static void font_smh(menu m, int n, menu_list ls[])
{
  update_submenu(n);
  menu_displaysubmenu(submenu);
}

menu fontmenu_create(fontmenu_handlep pp)
{
  unsigned char newpath[512], * d, e;
  unsigned char entry[80], entry2[80], last[32];
  int entryv = -1, lenlast = 1, /* size = 0, */l;
  _kernel_swi_regs regs;

  p = pp;
  _kernel_getenv("Font$Path", (char *)newpath, 512);
  if(strcmp((char*)newpath, (char*)oldpath) == 0)
    return fontmenu;
  strcpy((char*)oldpath, (char*)newpath);

  regs.r[2] = 0;
  * entry = * entry2 = * last = 0;
  do {
    regs.r[1] = (int)entry;
    regs.r[3] = 80;
    regs.r[4] = (int)entry2;
    regs.r[5] = 80;
    l = regs.r[2];
    if(_kernel_swi(0x60091, & regs, & regs))
      return 0;
    if(regs.r[2] != -1) {
      lenlast = 0;
      while((entry[lenlast] != '.') && (entry[lenlast] >= ' '))
	lenlast ++ ;
      entry[lenlast] = 0;
      if(stricmp((char *)entry, (char *)last)) {
	list[ ++ entryv] = l;
	sm[entryv] = 0;
	strcpy((char*)last, (char*)entry);
      } else {
	sm[entryv] = 1;
      }
    }
  } while(regs.r[2] != -1);
  list[ ++ entryv] = l;
  if(entryv > entries)
    fontmenu = menu_create("Font", entries = entryv, font_hnd, font_smh);
  if(submenu == 0)
    submenu = menu_create("Style", 16, 0, 0);
  menu_clear(fontmenu);
  for(l = 0; l < entryv; l ++) {
    regs.r[1] = (int)entry;
    regs.r[2] = list[l];
    regs.r[3] = 80;
    regs.r[4] = (int)entry2;
    regs.r[5] = 80;
    if(_kernel_swi(0x60091, & regs, & regs))
      return 0;
    d = entry;
    while((* d != '.') && (* d >= ' ')) d ++ ;
    e = * d;
    entry[11] = 0;
    * d = 0;
    menu_add(fontmenu, (char *)entry);
    if((e >= ' ') || (sm[l])) {
      menu_submenu(fontmenu, l, submenu);
      menu_subwarning(fontmenu, l);
    }
    if(strincmp((char *)entry, (char *)tickitem, strlen((char *)entry)) == 0)
      menu_tick(fontmenu, old_tick = l, 1);
  }
  return fontmenu;
}

void fontmenu_tickfont(unsigned char * c)
{
  if(c) {
    char * d = ((char *)fontmenu) + 28 + 12 + 20 + 4;
    int l, f;

    strcpy((char*)tickitem, (char*)c);
    if(old_tick > - 1)
      menu_tick(fontmenu, old_tick, 0);
    if(old_stick > - 1)
      menu_tick(submenu, old_stick, 0);
    old_tick = old_stick = -1;
    for(l = 0; l < entries; l ++) {
      f = strlen(d);
      if((strincmp((char *)c, d, f) == 0) && ((c[f] == '.') || (c[f] < ' '))) {
	menu_tick(fontmenu, old_tick = l, 1);
	update_submenu(l);
	return;
      }
      d += 24;
    }
  } else {
    if(old_tick > - 1)
      menu_tick(fontmenu, old_tick, 0);
    if(old_stick > - 1)
      menu_tick(submenu, old_stick, 0);
    old_tick = old_stick = -1;
    * tickitem = 0;
  }
}
